/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: hf.c,v 1.8 2002/11/11 10:55:04 kds Exp $

   OBEX client lib 

   Fixes:	Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

*/

#include <affix/config.h>

#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <libgen.h>

#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>
#include <signal.h>

#include <stdio.h>
#include <unistd.h>
#include <errno.h>

#include <affix/bluetooth.h>
#include <affix/btcore.h>

// SDP stuff
#include <affix/sdp.h>
#include <affix/sdpsrv.h>
#include <affix/sdpclt.h>

// OBEX stuff
//#include <obex.h>
//#include "obex_server.h"
//#include "obex_client.h"

/* SDP section */

#include <affix/bthfp.h>


hfp_t		*handle;


/* Client stuff
 */
void hfp_event(hfp_t *handle, int event)
{
}

void signal_handler(int sig)
{
	BTINFO("Sig handler : %d", sig);
	if (handle)
		hfp_cleanup(handle);
	BTINFO("Terminating on signal (%d) ...", sig);
	exit(0);
}

int accept_audio(void)
{
	struct sockaddr_affix	sa;
	int		fd, err, afd;
	socklen_t	len;

	fd = socket(PF_AFFIX, SOCK_RAW, BTPROTO_HCISCO);
	if (fd < 0)
		return fd;

	sa.bda = BDADDR_ANY;	/* accept connection from any device */
	err = bind(fd, (struct sockaddr*)&sa, sizeof(sa));
	if (err) {
		close(fd);
		return err;
	}

	err = listen(fd, 5);
	if (err) {
		close(fd);
		return err;
	}

	for(;;) {
		/* now sa will contain address info about connected device */
		afd = accept(fd, (struct sockaddr*)&sa, &len);
		if (afd < 0) {
			close(fd);
			return afd;
		}
		/*
		 * if (afd not satisfy you)
		 * 	continue;
		 */

		/* close server socket */
		close(fd);
		return afd;
	}
	return -1;
}


int main(int argc, char *argv[])
{
	int	err;

	btlog_mask = BTDEBUG_MODULE_HFP;

	/* install signal handlers to unregister services from SDP */
	signal(SIGCHLD, SIG_IGN);
	signal(SIGINT, signal_handler);		// CTRL-C
	signal(SIGTERM, signal_handler);
	signal(SIGABRT, signal_handler);
	signal(SIGQUIT, signal_handler);

	handle = hfp_init(hfp_event, 0);
	if (handle == NULL) {
		printf("Unable to create handle\n");
		return 1;
	}

	if (argv[1]) {
		struct sockaddr_affix	sa;
		
		if (strcasecmp(argv[1], "local") == 0)
			sa.bda = BDADDR_ANY;
		else {
			str2bda(&sa.bda, argv[1]);
		}
		sa.port = 0;
		err = hfp_connect(handle, &sa);
		if (err) {
			printf("Connection error\n");
			goto exit;
		}
		printf("Supported Features: %hx\n", handle->peerSupportedFeatures);

	} else {
		handle->name = "Hands Free";
		handle->SupportedFeatures = 0x1234;
		hfp_server_register(handle);
		pause();
	}
exit:	
	hfp_cleanup(handle);
	
	return 0;
}



