/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: modememu.c,v 1.3 2002/10/08 07:29:57 kds Exp $

   Modem Emulator

   Fixes:	Imre Deak <ext-imre.deak@nokia.com>
*/		

#include <syslog.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <time.h>
#include <string.h>
#include <ctype.h>

#define TIMEOUT 30000 

// reads from device, one character at a time with a fixed timeout
int read_line_nonblk(int fd, char *buf, int max_size)
{
        int     i;
        fd_set  fset_rd;
        struct  timeval tv;
        int     res;
 
        if (max_size < 1)
                return -1;
        for (i = 0; i < max_size - 1; i++)
        {
                FD_ZERO(&fset_rd);
                FD_SET(fd, &fset_rd);
                tv.tv_sec = 0;
                tv.tv_usec = TIMEOUT * 1000;
                if (select(fd + 1, &fset_rd, NULL, NULL, &tv) <= 0)
                        return i;
                if ((res = read(fd, &buf[i], 1)) <= 0)
                        return -1;
                buf[i + 1] = '\0';
		if (buf[i] == '\n' || buf[i] == '\r')
                        return i + 1;
        }
        return -1;
}

void nsleep(int sec, int nsec)
{
	struct timespec req;

	req.tv_sec = sec;
	req.tv_nsec = nsec;	/* 1msec */
	nanosleep(&req, NULL);
}

int write_to_modem(int fd, char *buf, int size)
{
	int i;

	for (i = 0; i < size; i++)
	{
		if (write(fd, buf++, 1) != 1)
			return -1;
		nsleep(0, 500000);
	}
	return 0;
}

#define CARRIER_STR "\r\nCARRIER\r\n"
#define CONNECT_STR "\r\nCONNECT 9600\r\n"
#define OK_STR "\r\nOK\r\n"

int main(void)
{
	static char 	buf[512];
	struct termios	term;

	openlog(NULL/*"btmodem"*/, 0, LOG_USER);
	if (tcgetattr(1, &term) != 0)
	{
		syslog(LOG_ERR, "tcgetattr failed.\n");
		goto failed;
	}
	cfmakeraw(&term);
	term.c_cflag |= CRTSCTS;
	if (tcsetattr(1, TCSANOW, &term) != 0)
	{
		syslog(LOG_ERR, "tcsetattr failed.\n");
		goto failed;
	}
	for (;;) {
		int i;
		int res = read_line_nonblk(0, buf, sizeof(buf));
		if (res <= 0) {
			syslog(LOG_ERR, "input failed.\n");
			goto failed;
		}
		syslog(LOG_DEBUG, "Got %s\n", buf);
		for (i = 0; i < (int)strlen(buf); i++)
			buf[i] = toupper(buf[i]);
		if (strncmp(buf, "AT", 2) == 0) {
			if (strncmp(buf, "ATD", 3) == 0 || strstr(buf, "DT") || strstr(buf, "DP"))
			{
				nsleep(1, 0);
				syslog(LOG_DEBUG, "Send %s %s", CARRIER_STR, CONNECT_STR);
				if (write_to_modem(1, CARRIER_STR, sizeof(CARRIER_STR) - 1) != 0 ||
				    write_to_modem(1, CONNECT_STR, sizeof(CONNECT_STR) - 1) != 0)
				{
					syslog(LOG_ERR, "write to modem failed.\n");
					goto failed;
				}
#if 0
				syslog(LOG_DEBUG, "Set CD\n");
				val = TIOCM_CD;
				if (ioctl(1, TIOCMBIS, &val) != 0)
				{
					syslog(LOG_ERR, "failed to set CD. errno=%d\n", errno);
					goto failed;
				}
#endif
				break;
			}
			else
			{
				syslog(LOG_DEBUG, "Send OK\\r");
				if (write_to_modem(1, OK_STR, sizeof(OK_STR) - 1) != 0)
				{
					syslog(LOG_ERR, "write to modem failed.\n");
					goto failed;
				}
			}
		}
		else if (strncmp(buf, "CLIENT", 6) == 0)
		{
			syslog(LOG_DEBUG, "Send CLIENTSERVER\n");
			write(1, "CLIENTSERVER", 12);
			break;
		}
		else
		{
			syslog(LOG_DEBUG, "Send ERROR\n");
			write(1, "\r\nERROR\r\n", 9);
		}
	}
	closelog();
	return 0;
failed:
	closelog();
	return -1;
}

